#region License

/* Copyright (c) 2000-2008 Marc Jacobi
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to 
 * deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */

#endregion

using System;
using System.Reflection;

using PostSharp.Laos;

namespace Jacobi.Diagnostics.PostSharp
{
    [Serializable]
    public class TraceContextAspectAttribute : OnMethodInvocationAspect
    {
        [NonSerialized]
        private MethodBase _methodBase;

        public override void RuntimeInitialize(MethodBase method)
        {
            _methodBase = method;
        }

        public override void OnInvocation(MethodInvocationEventArgs eventArgs)
        {
            using (AspectTraceContext ctx = new AspectTraceContext(_methodBase))
            {
                ctx.Initialize(eventArgs.Delegate.Target);
                AddParameters(ctx, eventArgs.GetArguments());
                ctx.TraceMethodEntry();

                try
                {
                    eventArgs.ReturnValue = eventArgs.Delegate.DynamicInvoke(eventArgs.GetArguments());

                    ctx.SetReturnValue(eventArgs.ReturnValue);
                }
                catch (Exception e)
                {
                    // TODO: expand non-serializable types

                    ctx.AddContextTo(e);

                    throw;
                }

            } // maintains stack and writes method exit and flushes writer.
        }

        private void AddParameters(AspectTraceContext ctx, object[] parameters)
        {
            int index = 0;

            foreach (ParameterInfo paramInfo in ctx.MethodBase.GetParameters())
            {
                ctx.AddParameterValue(parameters[index], paramInfo.Name);
                index++;
            }
        }
    }
}
