#region License

/* Copyright (c) 2000-2008 Marc Jacobi
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to 
 * deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */

#endregion

using System;
using Jacobi.Diagnostics.PostSharp;

namespace Jacobi.Diagnostics.TestApp
{
    [Serializable]
    internal class TraceAspectTest
    {
        public TraceAspectTest(string ctorName)
        {
            _name = ctorName;
        }

        private string _name;
        public string Name
        {
            get { return _name; }
        }


        [TraceContextAspect]
        public string PrintName(int numberOfTimes)
        {
            string result = String.Empty;

            for (int n = 0; n < numberOfTimes; n++)
            {
                result = ConcatName(result);
            }

            return result;
        }

        [TraceContextAspect]
        internal void ThrowException(string message)
        {
            throw new Exception(message);
        }

        [TraceContextAspect]
        private string ConcatName(string text)
        {
            return text + _name;
        }

        [TraceContextAspect]
        public string WithContext(string text)
        {
            AspectTraceContext ctx = AspectTraceContext.Current;

            MethodTracer.FormatTrace(System.Diagnostics.TraceEventType.Information, null, ctx, GetHashCode());

            return text;
        }

        [TraceContextAspect]
        public static string SomeStaticMethod()
        {
            return "SomeStaticMethod says hello";
        }
    }
}
