#region License

/* Copyright (c) 2000-2008 Marc Jacobi
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to 
 * deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */

#endregion

using System;
using System.Collections;
using System.Text;

namespace Jacobi.Diagnostics.Formatters
{
    /// <summary>
    /// Formats a Trace text for an <see cref="Exception"/>.
    /// </summary>
    internal class ExceptionTraceFormatter : TraceFormatterBase
    {
        public override string Format(TraceContext traceCtx, object objectToFormat)
        {
            return Format(traceCtx, (Exception)objectToFormat);
        }

        public string Format(TraceContext traceCtx, Exception exception)
        {
            StringBuilder traceText = new StringBuilder();

            while (exception != null)
            {
                FormatExceptionText(traceCtx, traceText, exception);

                exception = exception.InnerException;

                if (exception != null)
                {
                    traceText.Append(Environment.NewLine);
                    traceText.Append(Environment.NewLine);
                    traceText.Append("Inner Exception:");
                    traceText.Append(Environment.NewLine);
                }
            }

            return traceText.ToString();
        }

        private void FormatExceptionText(TraceContext traceCtx, StringBuilder traceText, Exception exception)
        {
            traceText.AppendFormat("{0} : \"{1}\"{2}",
                exception.GetType().FullName, exception.Message, Environment.NewLine);

            if (exception.Data != null && exception.Data.Count > 0)
            {
                foreach (DictionaryEntry entry in exception.Data)
                {
                    ITraceFormatter formatter = base.TraceFormatProvider.GetFormatter(entry.Value.GetType());
                    if (formatter != null)
                    {
                        traceText.AppendFormat("{0} = {1}", entry.Key, formatter.Format(traceCtx, entry.Value));
                    }
                    else
                    {
                        traceText.AppendFormat("{0} = \"{1}\"", entry.Key, entry.Value);
                        traceText.Append(Environment.NewLine);
                    }
                }
            }

            if (traceCtx != null)
            {
                ITraceFormatter formatter = base.TraceFormatProvider.GetFormatter(typeof(TraceContext));
                traceText.Append(formatter.Format(traceCtx, traceCtx));
            }
            else
            {
                traceText.Append(exception.StackTrace);
            }
        }
    }
}
