#region License

/* Copyright (c) 2000-2008 Marc Jacobi
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to 
 * deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */

#endregion

using System;
using System.Text;

namespace Jacobi.Diagnostics.Formatters
{
    /// <summary>
    /// Formats a Trace text for any <see cref="Object"/>.
    /// </summary>
    /// <remarks>If an object implements the <see cref="IFormattable"/>
    /// interface it will be called requesting the general format "G".</remarks>
    internal class ObjectTraceFormatter : TraceFormatterBase
    {
        public override string Format(TraceContext traceCtx, object objectToFormat)
        {
            StringBuilder traceText = new StringBuilder();

            // Use DumpObject to generate trace text
            IFormattable formattable = objectToFormat as IFormattable;

            if (formattable != null)
            {
                // general format
                traceText.Append(formattable.ToString("G",
                    System.Globalization.CultureInfo.InvariantCulture));
            }
            else
            {
                traceText.Append(objectToFormat);
            }
            
            traceText.Append(Environment.NewLine);

            return traceText.ToString();
        }
    }
}
