#region License

/* Copyright (c) 2000-2008 Marc Jacobi
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to 
 * deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */

#endregion

using System;
using System.Reflection;
using System.Text;

namespace Jacobi.Diagnostics.Formatters
{
    internal class TraceContextStackTraceFormatter : TraceFormatterBase
    {
        private int _maxDepth = 2;

        /// <summary>
        /// Formats the call stack for the specified <paramref name="traceCtx"/>.
        /// </summary>
        /// <param name="traceCtx">The current (calling method) <see cref="TraceContext"/>.</param>
        /// <param name="objectToFormat">Not used, can be null.</param>
        /// <returns>Returns a string containing the call stack.</returns>
        public override string Format(TraceContext traceCtx, object objectToFormat)
        {
            StringBuilder traceText = new StringBuilder();

            if (traceCtx == null)
            {
                traceCtx = objectToFormat as TraceContext;
            }

            ITraceFormatter formatter = base.TraceFormatProvider.GetFormatter(typeof(MethodBase));
            int depth = 0;

            // TODO: trace method context data

            while (traceCtx != null && depth < _maxDepth)
            {
                traceText.AppendFormat("at {0}", formatter.Format(traceCtx, traceCtx.MethodBase));

                traceCtx = traceCtx.CallingContext;

                if (traceCtx != null)
                {
                    traceText.Append(Environment.NewLine);
                }
            }

            return traceText.ToString();
        }
    }
}
