#region License

/* Copyright (c) 2000-2008 Marc Jacobi
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to 
 * deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */

#endregion

using System;
using System.Text;

namespace Jacobi.Diagnostics.Formatters
{
    /// <summary>
    /// Base class for Trace Formatter implementations.
    /// </summary>
    public abstract class TraceFormatterBase : ITraceFormatter
    {
        private ITraceFormatProvider _provider;
        /// <summary>
        /// Gets the trace format provider.
        /// </summary>
        /// <remarks>Derived classes can use this to query for formatter
        /// for contained types of the 'objectToFromat'.</remarks>
        protected ITraceFormatProvider TraceFormatProvider
        {
            get { return _provider; }
        }

        #region ITraceFormatter Members
        /// <summary>
        /// Initializes the trace formatter with the provider.
        /// </summary>
        /// <param name="provider">The trace format provider. Never null.</param>
        public void Initialize(ITraceFormatProvider provider)
        {
            Throw.IfArgumentNull(provider, "provider");

            _provider = provider;
        }

        /// <summary>
        /// Format a string based on the specified information.
        /// </summary>
        /// <param name="traceCtx">The trace context information. Can be null.</param>
        /// <param name="objectToFormat">The object to format.</param>
        /// <returns>Returns a string that contains the texted to be output.</returns>
        /// <remarks>
        /// It is considered best practice to also provide a typed overload for
        /// the Format method where the <paramref name="objectToFormat"/> is not
        /// of type <see cref="System.Object"/> but of the supported type.
        /// </remarks>
        public abstract string Format(TraceContext traceCtx, object objectToFormat);

        #endregion
    }
}
