#region License

/* Copyright (c) 2000-2008 Marc Jacobi
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to 
 * deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */

#endregion

using System;
using System.Diagnostics;

namespace Jacobi.Diagnostics
{
    /// <summary>
    /// Implemented by components that know how to format a trace text
    /// for a certain type of object.
    /// </summary>
    /// <remarks>
    /// TraceFormatter implementations are registered with a Type that
    /// identifies the type they support to format trace texts for.
    /// Trace formatter implementations should be stateless (except for
    /// a reference to the ITraceFormatProvider).
    /// </remarks>
    public interface ITraceFormatter
    {
        /// <summary>
        /// Initializes the trace formatter with the provider.
        /// </summary>
        /// <param name="provider">The trace format provider. Never null.</param>
        void Initialize(ITraceFormatProvider provider);
        /// <summary>
        /// Format a string based on the specified information.
        /// </summary>
        /// <param name="traceCtx">The trace context information. Can be null.</param>
        /// <param name="objectToFormat">The object to format.</param>
        /// <returns>Returns a string that contains the text to be output.</returns>
        string Format(TraceContext traceCtx, object objectToFormat);
    }
}
