#region License

/* Copyright (c) 2000-2008 Marc Jacobi
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to 
 * deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */

#endregion

using System;

namespace Jacobi
{
    public static class Check
    {
        /// <summary>
        /// Returns an indiaction if the parameter <paramref name="value"/> is out of range.
        /// </summary>
        /// <typeparam name="T">The parameter data type.</typeparam>
        /// <param name="value">The parameter value.</param>
        /// <param name="minValue">The parameter's minimal value.</param>
        /// <param name="maxValue">The parameter's maximal value.</param>
        /// <returns>Returns true if the parameter <paramref name="value"/> is not in range
        /// of <paramref name="minValue"/> and <paramref name="maxValue"/> (exclusive)
        /// otherwise false is returned.</returns>
        public static bool IsOutOfRange<T>(IComparable<T> value, T minValue, T maxValue)
        {
            return (value.CompareTo(minValue) < 0 || value.CompareTo(maxValue) > 0);
        }

        /// <summary>
        /// Returns an indiaction if the parameter <paramref name="value"/> is null or <see cref="DbNull"/>.
        /// </summary>
        /// <param name="value">The parameter value.</param>
        /// <returns>
        /// Returns true if the parameter <paramref name="value"/> is null or <see cref="DbNull"/>
        /// otherwise false is returned.
        /// </returns>
        public static bool IsNullOrDbNull(object value)
        {
            return (value == null || value == DBNull.Value);
        }
    }
}
